package com.hzya.frame.plugin.lets.plugin.sales;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.plugin.lets.constant.OverallConstant;
import com.hzya.frame.plugin.lets.constant.ProfilesActiveConstant;
import com.hzya.frame.plugin.lets.dao.IOnlyAfterSalesBDao;
import com.hzya.frame.plugin.lets.dao.IOnlyAfterSalesHDao;
import com.hzya.frame.plugin.lets.entity.OnlyAfterSalesBEntity;
import com.hzya.frame.plugin.lets.entity.OnlyAfterSalesHEntity;
import com.hzya.frame.plugin.lets.ofsvo.QueryReturnOrderRequestVO;
import com.hzya.frame.plugin.lets.u8cdto.ArapDjzbVO;
import com.hzya.frame.plugin.lets.u8cdto.BillVO;
import com.hzya.frame.plugin.lets.util.QueryU8CEntityUtil;
import com.hzya.frame.plugin.lets.util.SaveOrUpdateBusinessLogUtil;
import com.hzya.frame.plugin.lets.util.pushData.PushU8CByApiCode;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.ttxofs.basics.ReturnMessageBasics;
import com.hzya.frame.ttxofs.dto.InterfaceParamDto;
import com.hzya.frame.ttxofs.dto.returngoodordersearch.RerturnGoodsOrderSearchData;
import com.hzya.frame.ttxofs.dto.returngoodordersearch.RerturnGoodsOrderSearchDetails;
import com.hzya.frame.ttxofs.dto.returngoodordersearch.RerturnGoodsOrderSearchHeader;
import com.hzya.frame.ttxofs.dto.returngoodordersearch.RerturnGoodsOrderSearchJsonRootBean;
import com.hzya.frame.ttxofs.service.OfsUnifiedService;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * add by zyd
 * O售后订单(仅退款) -> U8C红字应收单
 *
 * returnType:0 不退货
 * exchangeType:0 不换货
 * refundInd:1 退款
 *
 * 按天推送
 *
 * @Date：2024/9/21 17:19
 */
public class RefundOnlyPluginInitializer extends PluginBaseEntity {

    Logger logger = LoggerFactory.getLogger(RefundOnlyPluginInitializer.class);

    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "執行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "執行銷毀方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "sales.RefundOnlyPluginInitializer";
    }

    @Override
    public String getPluginName() {
        return "丽知：售后订单(仅退款)--->U8C红字应收单";
    }

    @Override
    public String getPluginLabel() {
        return "丽知：售后订单(仅退款)--->U8C红字应收单";
    }

    @Override
    public String getPluginType() {
        return "3";
    }


    @Autowired
    private OfsUnifiedService ofsUnifiedService;
    @Autowired
    private IIntegrationTaskLivingDetailsDao iIntegrationTaskLivingDetailsDao;
    @Autowired
    private PushU8CByApiCode pushU8CByApiCode;
    @Autowired
    private SaveOrUpdateBusinessLogUtil saveOrUpdateBusinessLogUtil;
    @Autowired
    private QueryU8CEntityUtil queryU8CEntityUtil;

    @Autowired
    private IOnlyAfterSalesHDao onlyAfterSalesHDao;
    @Autowired
    private IOnlyAfterSalesBDao onlyAfterSalesBDao;


    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        try {
            logger.info("调用：" + getPluginName() + "-插件");
            String prod = "prod";
            String param = String.valueOf(requestJson.get("param"));
            if (requestJson != null && ProfilesActiveConstant.TYPE_DATE.equals(requestJson.get("type"))) {
                //按日期
                if (param != null && !"".equals(param)) {
                    String[] split = param.split("/");
                    if (!(split.length == 2)) {
                        Assert.state(false, "时间格式传递不正确");
                    }
                    Assert.notNull(split[0], "开始时间不能为空");
                    Assert.notNull(split[1], "结束时间不能为空");
                    start(split[0], split[1]);
                }
            } else if (requestJson != null && ProfilesActiveConstant.TYPE_VBILLCODE.equals(requestJson.get("type"))) {
                //按单号
                if (param != null && !"".equals(param)) {
                    start(param);
                }
            } else if (ProfilesActiveConstant.TYPE_DETAIL_ERROR.equals(requestJson.get("type"))) {
                String integrationTaskLivingDetails = (String) requestJson.get("integration_task_living_details_id");
                IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = saveOrUpdateBusinessLogUtil.queryIntegrationTaskLivingDetailsEntity(integrationTaskLivingDetails);
                start(integrationTaskLivingDetailsEntity.getRootAppPk());
            } else if (ProfilesActiveConstant.LETS_PROFILES_ACTIVE.equals(prod)) {
                //默认
                start();
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("executeBusiness方法抛出异常", e);
        }
        return null;
    }

    public void start(){
        try {
            //获取当前时间
            Date currentDate = new Date();
            Date startTime = DateUtil.offset(currentDate, DateField.MINUTE, -10);
            Date endTime = DateUtil.offset(currentDate, DateField.MINUTE, -3);
            String startTimeStr = DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss");
            String endTimeStr = DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss");

            QueryReturnOrderRequestVO queryReturnOrderRequestVO = new QueryReturnOrderRequestVO();
            queryReturnOrderRequestVO.setRefundedAt_start(startTimeStr);
            queryReturnOrderRequestVO.setRefundedAt_end(endTimeStr);
//            queryOfsOnlyReturnOrders(queryReturnOrderRequestVO);


        }catch (Exception e){
            logger.error("丽知：售后订单(仅退款)--->U8C红字应收单,start()方法报错："+e);
        }
    }

    /**
     * @param ofsCode 售后订单号
     */
    public void start(String ofsCode){
        try {
            QueryReturnOrderRequestVO queryReturnOrderRequestVO = new QueryReturnOrderRequestVO();
            queryReturnOrderRequestVO.setCode(ofsCode);
            List<RerturnGoodsOrderSearchData> returnGoodsOrders = queryOfsOnlyReturnOrders(queryReturnOrderRequestVO);
            if (returnGoodsOrders == null || returnGoodsOrders.size() == 0) {
                Assert.state(false, "丽知：售后订单(仅退款):{} --->U8C红字应收单，未查询到OFS售后订单(仅退款)", ofsCode);
            }

            //底表存数据库
            saveOrUpdate(returnGoodsOrders);
            //过滤日志
            List<RerturnGoodsOrderSearchData> filterReturnGoodsOrders = filterData(returnGoodsOrders);
            if(filterReturnGoodsOrders==null||filterReturnGoodsOrders.size()==0){
                Assert.state(false, "丽知：售后订单(仅退款):{} --->U8C红字应收单，该售后订单(仅退款)已被推送成功，请勿重新推送", ofsCode);
            }
            //推送
//            implement(filterReturnGoodsOrders);


        }catch (Exception e){
            logger.error("丽知：售后订单(仅退款)--->U8C红字应收单,start(String ofsCode)方法报错："+e);
        }
    }
    public void start(String startTime, String endTime){
        try {
            Date business_start = DateUtil.parse(startTime);
            Date business_end = DateUtil.parse(endTime);
            String start = DateUtil.format(business_start, "yyyy-MM-dd") + " 00:00:00";
            String end = DateUtil.format(business_end, "yyyy-MM-dd") + " 23:59:59";

            QueryReturnOrderRequestVO queryReturnOrderRequestVO = new QueryReturnOrderRequestVO();
            queryReturnOrderRequestVO.setRefundedAt_start(start);
            queryReturnOrderRequestVO.setRefundedAt_end(end);
//            queryOfsOnlyReturnOrders(queryReturnOrderRequestVO);


        }catch (Exception e){
            logger.error("丽知：售后订单(仅退款)--->U8C红字应收单,start(String startTime, String endTime)方法报错："+e);
        }
    }

    /**
     * 过滤成功日志
     */
    public List<RerturnGoodsOrderSearchData> filterData(List<RerturnGoodsOrderSearchData> returnGoodsOrders){
        List<RerturnGoodsOrderSearchData> filterReturnGoodsOrders=new ArrayList<>();
        for (RerturnGoodsOrderSearchData returnGoodsOrder : returnGoodsOrders) {
            RerturnGoodsOrderSearchHeader header = returnGoodsOrder.getHeader();
            String rootAppPk=header.getCode();
            boolean isExis = true;
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setRootAppPk(rootAppPk);
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
            integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
            List<IntegrationTaskLivingDetailsEntity> integrationTaskLivingDetailsEntities = iIntegrationTaskLivingDetailsDao.query(integrationTaskLivingDetailsEntity);
            if (integrationTaskLivingDetailsEntities == null || integrationTaskLivingDetailsEntities.size() == 0) {
                isExis = false;
            }
            if (!isExis) {
                filterReturnGoodsOrders.add(returnGoodsOrder);
            }
        }
        return filterReturnGoodsOrders;
    }

    /**
     * 具体实现推送U8C红字应收单
     *
     * 公司+店铺+仓库+SKU+收发类别+部门 分组
     *
     * 因为优惠金额在表头，先按单据便利一变，具体分摊金额拆到每行，在去汇总
     *
     */
    public void implement(List<RerturnGoodsOrderSearchData> filterReturnGoodsOrders){

        //分摊后的明细
        List<RerturnGoodsOrderSearchDetails> ofsDetails=new ArrayList<>();




//        for (RerturnGoodsOrderSearchData returnGoodsOrder : filterReturnGoodsOrders) {
//            System.out.println(returnGoodsOrder);
//
//            String mapStr = "";
//            String format = "";//暂时取创建时间，refundAt没返回
//            String response = "";
//            String ofsCode = returnGoodsOrder.getHeader().getCode();
//            String ofsId = returnGoodsOrder.getHeader().getId();
//
//            try {
//
//                ArapDjzbVO arapDjzbVO = new ArapDjzbVO();
//                //主表 ， 在表头先
//                RerturnGoodsOrderSearchHeader header = returnGoodsOrder.getHeader();
//                ArapDjzbVO.Parentt parentvo = new ArapDjzbVO.Parentt();
//
//                //djrq 单据日期
//                format = returnGoodsOrder.getHeader().getCreated();//暂时取创建时间，refundAt没返回
//                parentvo.setDjrq(format.substring(0,10));
//                //dwbm 公司
//                parentvo.setDwbm(header.getCompanyCode());
//                //lrr 录入人
//                parentvo.setLrr(OverallConstant.getOverAllValue("u8cApiZdrCode"));
//
//                //wldx 往来对象标识 (0 客户 2部门 3业务员)
//                //hbbm 客商
//                //deptid 部门
//                //ywybm 业务员编码
//
//
//                //shr 审核人
//                parentvo.setShr(OverallConstant.getOverAllValue("u8cApiZdrCode"));
//                //shrq 审核日
//                parentvo.setShrq(format.substring(0,10));
//                //scomment 备注      returnNote 客服备注    sellerNote 卖家备注
//                parentvo.setScomment(header.getReturnNote());
//
//
//                //zyx1店铺
//                parentvo.setZyx1(header.getStoreCode());
//                //zyx2来源平台
//                parentvo.setZyx2(header.getSourcePlatformCode());
//                //zyx3 收发类别 ????
//
//
//                //子表
//                List<ArapDjzbVO.Childrenn> children=new ArrayList<>();
//                List<RerturnGoodsOrderSearchDetails> details = returnGoodsOrder.getDetails();
//                for (RerturnGoodsOrderSearchDetails ofsDetail : details) {
//                    ArapDjzbVO.Childrenn childrenn = new ArapDjzbVO.Childrenn();
//
//                    //cinventoryid 存货
//
//                    //jfbbje jfybje 借方本币金额/借方原币金额
//                    //jfbbsj jfybsj存货 借方本币税金/借方原币税金
//                    //wbfbbje jfybwsje 借方本币无税金额/借方原币无税金额
//
//
//
//                    //zyx4 平台优惠
//                    //zyx5 支付优惠
//                    //zyx6 达人优惠
//                    //zyx7 商家优惠
//                    children.add(childrenn);
//                }
//                arapDjzbVO.setParentvo(parentvo);
//                arapDjzbVO.setChildren(children);
//                System.out.println(arapDjzbVO);
//                mapStr="{\"djzbvo\":["+ JSONUtil.toJsonStr(arapDjzbVO) +"]}";
//
//
//
//
//            }catch (Exception e){
//                e.printStackTrace();
//                logger.error("推送U8C，丽知：售后订单(仅退款)--->U8C红字应收单失败，失败原因：" + e);
//                //失败
//            }
//
//        }
        
    }

    /**
     * ofs仅退款数据，存中台mysql数据库
     * 表头表体分别存，忽略大小写 BeanUtil.copyPropreties(a,b,Copyoptions.create().ignoreCase())
     * @param returnGoodsOrders
     */
    public void saveOrUpdate(List<RerturnGoodsOrderSearchData> returnGoodsOrders){
//        List<OnlyAfterSalesHEntity> allH=new ArrayList<>();
//        List<OnlyAfterSalesBEntity> allB=new ArrayList<>();
        for (RerturnGoodsOrderSearchData returnGoodsOrder : returnGoodsOrders) {
            RerturnGoodsOrderSearchHeader header = returnGoodsOrder.getHeader();
            OnlyAfterSalesHEntity onlyAfterSalesHEntity = new OnlyAfterSalesHEntity();
            BeanUtil.copyProperties(header,onlyAfterSalesHEntity, CopyOptions.create().ignoreCase());
//            allH.add(onlyAfterSalesHEntity);
            onlyAfterSalesHEntity.setId(null);
            onlyAfterSalesHEntity.setOfsid(header.getId());
            OnlyAfterSalesHEntity save = onlyAfterSalesHDao.save(onlyAfterSalesHEntity);
            String HId = save.getId();

            List<RerturnGoodsOrderSearchDetails> details = returnGoodsOrder.getDetails();
            for (RerturnGoodsOrderSearchDetails detail : details) {
                OnlyAfterSalesBEntity onlyAfterSalesBEntity = new OnlyAfterSalesBEntity();
                BeanUtil.copyProperties(detail,onlyAfterSalesBEntity, CopyOptions.create().ignoreCase());
//                allB.add(onlyAfterSalesBEntity);
                onlyAfterSalesBEntity.setId(null);
                onlyAfterSalesBEntity.setHid(HId);
                onlyAfterSalesBEntity.setOfsid(detail.getId());
                onlyAfterSalesBDao.save(onlyAfterSalesBEntity);
            }
        }
    }


    /**
     * 查询ofs仅退款订单  销售退货单查询（ofs.returnOrder. search)
     *
     * 查询条件：refundedAt_start、refundedAt_end、code
     * returnType:0 不退货
     * exchangeType:0 不换货
     * refundInd:1 退款
     */
    public List<RerturnGoodsOrderSearchData> queryOfsOnlyReturnOrders(QueryReturnOrderRequestVO queryReturnOrderRequestVO) throws Exception {
        Long pageNo = 1L;

        List<RerturnGoodsOrderSearchData> all=new ArrayList<>();

        InterfaceParamDto interfaceParamDto = new InterfaceParamDto();
        interfaceParamDto.setApi("ofs.returnOrder.search");
        queryReturnOrderRequestVO.setClientCode("LETS");
        //100订单池 500待同步 510已同步 900售后成功 400待合单 800已入库 999作废 700收货中 0待完善
//        queryReturnOrderRequestVO.setStatus("900");
        queryReturnOrderRequestVO.setPageNo(pageNo);
        queryReturnOrderRequestVO.setPageSize(500L);
        interfaceParamDto.setData(JSON.toJSONString(queryReturnOrderRequestVO));

        RerturnGoodsOrderSearchJsonRootBean rerturnGoodsOrderSearchJsonRootBean =(RerturnGoodsOrderSearchJsonRootBean) ofsUnifiedService.unified(interfaceParamDto);
        if (rerturnGoodsOrderSearchJsonRootBean.getData() == null || rerturnGoodsOrderSearchJsonRootBean.getData().size() == 0) {
            return null;
        }
        all.addAll(rerturnGoodsOrderSearchJsonRootBean.getData());

        int count = rerturnGoodsOrderSearchJsonRootBean.getData().size();
        while (count >= 500) {//>=500,可能还有下一页。
            //还有下一页
            pageNo++;
            queryReturnOrderRequestVO.setPageNo(pageNo);
            interfaceParamDto.setData(JSON.toJSONString(queryReturnOrderRequestVO));
            RerturnGoodsOrderSearchJsonRootBean rerturnGoodsOrderSearchJsonRootBean1 =(RerturnGoodsOrderSearchJsonRootBean) ofsUnifiedService.unified(interfaceParamDto);
            all.addAll(rerturnGoodsOrderSearchJsonRootBean1.getData());
            count = rerturnGoodsOrderSearchJsonRootBean1.getData().size();
        }
        return all;
    }


}