package com.hzya.frame.plugin.lets.util;

import cn.hutool.core.lang.Assert;
import com.hzya.frame.beanutil.BeanUtil;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.DetailsDto;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDto;
import com.hzya.frame.ttxofs.dto.returngoodordersearch.RerturnGoodsOrderSearchData;
import com.hzya.frame.ttxofs.dto.returngoodordersearch.RerturnGoodsOrderSearchDetails;
import com.hzya.frame.ttxofs.dto.returngoodordersearch.RerturnGoodsOrderSearchHeader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 处理OFS售后订单，优惠金额分摊逻辑
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.plugin.lets.util
 * @Project：kangarooDataCenterV3
 * @name：AmountAllocationUtil
 * @Date：2024/9/21 18:19
 * @Filename：AmountAllocationUtil
 */
@Component
public class OfsOrderAfterSalesAmountAllocationUtil {

    Logger logger = LoggerFactory.getLogger(OfsOrderAfterSalesAmountAllocationUtil.class);

    /**
     * TOC销售金额分摊
     *
     * @param rerturnGoodsOrderSearchData OFS售后订单
     * @author liuyang
     */
    public void tocSalesAmountAllocation(RerturnGoodsOrderSearchData rerturnGoodsOrderSearchData) throws Exception {
        logger.info("==TOC退货金额分摊开始==");

        Assert.notNull(rerturnGoodsOrderSearchData, "OFS售后订单对象不能为空");
        RerturnGoodsOrderSearchHeader header = rerturnGoodsOrderSearchData.getHeader();
        List<RerturnGoodsOrderSearchDetails> detailsOld = rerturnGoodsOrderSearchData.getDetails();

        //拷贝一份List集合，并移除没有实付金额的原始
        //TODO 暂时取零售价作为分类的标准，需要和万万确认一下
        List<RerturnGoodsOrderSearchDetails> rerturnGoodsOrderSearchDetails = copyDetailsDto(detailsOld);
        rerturnGoodsOrderSearchDetails.removeIf(dto -> dto.getMsrPrice() == null || !"0".equals(new BigDecimal(dto.getMsrPrice()).stripTrailingZeros().toPlainString()));

        //计算分摊百分比
        calculatePercentage(rerturnGoodsOrderSearchData);

        String platformDiscounts = header.getPlatformDiscounts();
        String merchantDiscounts = header.getMerchantDiscounts();
        String expertDiscounts = header.getExpertDiscounts();
        String payDiscounts = header.getPayDiscounts();

        BigDecimal targetPlatformDiscounts = null;
        BigDecimal targetMerchantDiscounts = null;
        BigDecimal targetExpertDiscounts = null;
        BigDecimal targetPayDiscounts = null;

        //这4个优惠字段，可能没有，也有可能为0！
        if (platformDiscounts != null && !"".equals(platformDiscounts) && !"0".equals(new BigDecimal(platformDiscounts).stripTrailingZeros().toPlainString())) {
            targetPlatformDiscounts = new BigDecimal(platformDiscounts);
        }
        if (merchantDiscounts != null && !"".equals(merchantDiscounts) && !"0".equals(new BigDecimal(merchantDiscounts).stripTrailingZeros().toPlainString())) {
            targetMerchantDiscounts = new BigDecimal(merchantDiscounts);
        }
        if (expertDiscounts != null && !"".equals(expertDiscounts) && !"0".equals(new BigDecimal(expertDiscounts).stripTrailingZeros().toPlainString())) {
            targetExpertDiscounts = new BigDecimal(expertDiscounts);
        }
        if (payDiscounts != null && !"".equals(payDiscounts) && !"0".equals(new BigDecimal(payDiscounts).stripTrailingZeros().toPlainString())) {
            targetPayDiscounts = new BigDecimal(payDiscounts);
        }

        /**
         * OFS销售订单明细行，没有分摊金额的几种情况：
         * 1.对应的OFS销售订单表头某个优惠字段为空或为0
         * 2.明细行没有实付金额或者为0，所以没有分摊比例，不参与分摊
         */
        BigDecimal totalShareTargetPlatformDiscounts = new BigDecimal("0");
        BigDecimal totalShareTargetMerchantDiscounts = new BigDecimal("0");
        BigDecimal totalShareTargetExpertDiscounts = new BigDecimal("0");
        BigDecimal totalShareTargetPayDiscounts = new BigDecimal("0");

        //分摊金额=分摊比例*优惠金额
        for (int i = 0; i < rerturnGoodsOrderSearchDetails.size(); i++) {
            RerturnGoodsOrderSearchDetails rerturnGoodsOrderSearchDetails1 = rerturnGoodsOrderSearchDetails.get(i);
            //分摊比例
            BigDecimal sharingRatio = rerturnGoodsOrderSearchDetails1.getSharingRatio();

            //这里应该先8为小数，实际汇总到出库单时，再四舍五入两位小数
            BigDecimal shareTargetPlatformDiscounts = null;
            BigDecimal shareTargetMerchantDiscounts = null;
            BigDecimal shareTargetExpertDiscounts = null;
            BigDecimal shareTargetPayDiscounts = null;
            if (sharingRatio != null && targetPlatformDiscounts != null) {
                shareTargetPlatformDiscounts = targetPlatformDiscounts.multiply(sharingRatio).setScale(8, BigDecimal.ROUND_HALF_UP);
                rerturnGoodsOrderSearchDetails1.setOriginalTargetPlatformDiscounts(targetPlatformDiscounts);
                rerturnGoodsOrderSearchDetails1.setShareTargetPlatformDiscounts(shareTargetPlatformDiscounts);

                totalShareTargetPlatformDiscounts = totalShareTargetPlatformDiscounts.add(shareTargetPlatformDiscounts);
            }
            if (sharingRatio != null && targetMerchantDiscounts != null) {
                shareTargetMerchantDiscounts = targetMerchantDiscounts.multiply(sharingRatio).setScale(8, BigDecimal.ROUND_HALF_UP);
                rerturnGoodsOrderSearchDetails1.setOriginalTargetMerchantDiscounts(targetMerchantDiscounts);
                rerturnGoodsOrderSearchDetails1.setShareTargetMerchantDiscounts(shareTargetMerchantDiscounts);

                totalShareTargetMerchantDiscounts = totalShareTargetPlatformDiscounts.add(shareTargetMerchantDiscounts);
            }
            if (sharingRatio != null && targetExpertDiscounts != null) {
                shareTargetExpertDiscounts = targetExpertDiscounts.multiply(sharingRatio).setScale(8, BigDecimal.ROUND_HALF_UP);
                rerturnGoodsOrderSearchDetails1.setOriginalTargetExpertDiscounts(targetExpertDiscounts);
                rerturnGoodsOrderSearchDetails1.setShareTargetExpertDiscounts(shareTargetExpertDiscounts);

                totalShareTargetExpertDiscounts = totalShareTargetPlatformDiscounts.add(shareTargetExpertDiscounts);
            }
            if (sharingRatio != null && targetPayDiscounts != null) {
                shareTargetPayDiscounts = targetPayDiscounts.multiply(sharingRatio).setScale(8, BigDecimal.ROUND_HALF_UP);
                rerturnGoodsOrderSearchDetails1.setOriginalTargetPayDiscounts(targetPayDiscounts);
                rerturnGoodsOrderSearchDetails1.setShareTargetPayDiscounts(shareTargetPayDiscounts);

                totalShareTargetPayDiscounts = totalShareTargetPlatformDiscounts.add(shareTargetPayDiscounts);
            }

            //如果是最后一行，则把尾差进行追加
            if (i == rerturnGoodsOrderSearchDetails.size() - 1) {
                if (!"0".equals(totalShareTargetPlatformDiscounts.stripTrailingZeros().toPlainString()) && rerturnGoodsOrderSearchDetails1.getShareTargetPlatformDiscounts() != null) {
                    //原始优惠金额-sum（分摊金额）
                    BigDecimal subtract = targetPlatformDiscounts.subtract(totalShareTargetPlatformDiscounts);
                    BigDecimal shareTargetPlatformDiscounts1 = rerturnGoodsOrderSearchDetails1.getShareTargetPlatformDiscounts();
                    rerturnGoodsOrderSearchDetails1.setShareTargetPlatformDiscounts(shareTargetPlatformDiscounts1.add(subtract));
                }
                if (!"0".equals(totalShareTargetMerchantDiscounts.stripTrailingZeros().toPlainString()) && rerturnGoodsOrderSearchDetails1.getShareTargetMerchantDiscounts() != null) {
                    //原始优惠金额-sum（分摊金额）
                    BigDecimal subtract = targetMerchantDiscounts.subtract(totalShareTargetMerchantDiscounts);
                    BigDecimal shareTargetPlatformDiscounts1 = rerturnGoodsOrderSearchDetails1.getShareTargetPlatformDiscounts();
                    rerturnGoodsOrderSearchDetails1.setShareTargetPlatformDiscounts(shareTargetPlatformDiscounts1.add(subtract));
                }
                if (!"0".equals(totalShareTargetExpertDiscounts.stripTrailingZeros().toPlainString()) && rerturnGoodsOrderSearchDetails1.getShareTargetExpertDiscounts() != null) {
                    BigDecimal subtract = targetExpertDiscounts.subtract(totalShareTargetExpertDiscounts);
                    BigDecimal shareTargetExpertDiscounts1 = rerturnGoodsOrderSearchDetails1.getShareTargetExpertDiscounts();
                    rerturnGoodsOrderSearchDetails1.setShareTargetExpertDiscounts(shareTargetExpertDiscounts1.add(subtract));
                }
                if (!"0".equals(totalShareTargetPayDiscounts.stripTrailingZeros().toPlainString()) && rerturnGoodsOrderSearchDetails1.getShareTargetPayDiscounts() != null) {
                    BigDecimal subtract = targetPayDiscounts.subtract(totalShareTargetPayDiscounts);
                    BigDecimal shareTargetPayDiscounts1 = rerturnGoodsOrderSearchDetails1.getShareTargetPayDiscounts();
                    rerturnGoodsOrderSearchDetails1.setShareTargetPayDiscounts(shareTargetPayDiscounts1.add(subtract));
                }
            }

            //拷贝回到原始的集合
            copyDetailsDtoOld(rerturnGoodsOrderSearchDetails, detailsOld);
        }
        logger.info("==TOC退货金额分摊结束==");
    }

    /**
     * 计算百分比
     * 分摊比例=零售价/sum(零售价)
     *
     * @param rerturnGoodsOrderSearchData OFS售后订单
     * @author liuyang
     */
    private void calculatePercentage(RerturnGoodsOrderSearchData rerturnGoodsOrderSearchData) throws Exception {
        Assert.notNull(rerturnGoodsOrderSearchData, "headerDetailsDto2不能为空");
        RerturnGoodsOrderSearchHeader header = rerturnGoodsOrderSearchData.getHeader();
        List<RerturnGoodsOrderSearchDetails> details = rerturnGoodsOrderSearchData.getDetails();

        BigDecimal sumActualPaymentAmountBigDecimal = sumActualPaymentAmount(details, header.getCode());
        for (int i = 0; i < details.size(); i++) {
            RerturnGoodsOrderSearchDetails rerturnGoodsOrderSearchDetails = details.get(i);

            String msrPrice = rerturnGoodsOrderSearchDetails.getMsrPrice();
            BigDecimal msrPriceBigDecimal = new BigDecimal(msrPrice);
            //如果实付金额为0则不进行分摊！
            if (!"0".equals(msrPriceBigDecimal.stripTrailingZeros().toPlainString())) {
                //得到分摊比例
                BigDecimal percentageBigDecimal = msrPriceBigDecimal.divide(sumActualPaymentAmountBigDecimal, 20, BigDecimal.ROUND_HALF_UP).setScale(8, BigDecimal.ROUND_HALF_UP);
                rerturnGoodsOrderSearchDetails.setSharingRatio(percentageBigDecimal);
            }
        }
    }

    /**
     * 累计这一单对应的实付金额
     *
     * @param details OFS销售订单明细行
     * @param code    OFS售后订单号
     * @author liuyang
     */
    private BigDecimal sumActualPaymentAmount(List<RerturnGoodsOrderSearchDetails> details, String code) throws Exception {
        Assert.notNull(details, "details1不能为空");
        Assert.notNull(code, "code不能为空");

        BigDecimal actualPaymentAmountBigDecimal = new BigDecimal("0");
        for (int i = 0; i < details.size(); i++) {
            RerturnGoodsOrderSearchDetails rerturnGoodsOrderSearchDetails = details.get(i);

            String msrPrice = rerturnGoodsOrderSearchDetails.getMsrPrice();
            if (msrPrice == null || "".equals(msrPrice.trim())) {
                Assert.state(false, "售后订单零售价不能为空 OFS售后订单号：{}", code);
            }
            BigDecimal msrPriceBigDecimal = new BigDecimal(msrPrice);
            actualPaymentAmountBigDecimal = actualPaymentAmountBigDecimal.add(msrPriceBigDecimal);
        }
        if ("0".equals(actualPaymentAmountBigDecimal.stripTrailingZeros().toPlainString())) {
            Assert.state(false, "售后订单零售价(合并后)不能为0 OFS售后订单号：{}", code);
        }
        return actualPaymentAmountBigDecimal;
    }

    /**
     * 集合复制
     *
     * @param details 旧OFS售后订单明细行
     * @author liuyang
     */
    private List<RerturnGoodsOrderSearchDetails> copyDetailsDto(List<RerturnGoodsOrderSearchDetails> details) throws Exception {
        List<RerturnGoodsOrderSearchDetails> newDetails = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            RerturnGoodsOrderSearchDetails rerturnGoodsOrderSearchDetails = details.get(i);

            RerturnGoodsOrderSearchDetails newDetailsDto1 = new RerturnGoodsOrderSearchDetails();
            BeanUtil.copyPropertiesV2(rerturnGoodsOrderSearchDetails, newDetailsDto1);
            newDetails.add(newDetailsDto1);
        }
        return newDetails;
    }

    /**
     * 拷贝到原集合中
     *
     * @param newDetails 新OFS销售订单明细行
     * @param detailsOld 旧OFS销售订单明细行
     * @author liuyang
     */
    private void copyDetailsDtoOld(List<RerturnGoodsOrderSearchDetails> newDetails, List<RerturnGoodsOrderSearchDetails> detailsOld) throws Exception {
        if (newDetails != null && detailsOld != null) {
            for (int i = 0; i < detailsOld.size(); i++) {
                RerturnGoodsOrderSearchDetails rerturnGoodsOrderSearchDetails = detailsOld.get(i);
                for (int j = 0; j < newDetails.size(); j++) {
                    RerturnGoodsOrderSearchDetails rerturnGoodsOrderSearchDetails1 = newDetails.get(j);
                    if (rerturnGoodsOrderSearchDetails.getId().equals(rerturnGoodsOrderSearchDetails1.getId())) {
                        //把分摊后的金额拷贝到旧的OFS销售订单集合中
                        BeanUtil.copyPropertiesV2(rerturnGoodsOrderSearchDetails1, rerturnGoodsOrderSearchDetails);
                    }
                }
            }
        } else {
            logger.error("copyDetailsDtoOld方法newDetails或者detailsOld为空！");
        }
    }
}